<?php 
class controller_component_forum extends component {
	private $errors = array();

	public function action_index() {
		$this->load_model('component', 'users');
		
		$data = array();
		
		if (isset($this->url['params']['page']) && $this->url['params']['page'] > 0) {
			$page = (int)$this->url['params']['page'];
		} else {
			$page = 1;
		}
		
		$start = $this->component_config['topics_per_page'] * ($page - 1);
		$end = $page * $this->component_config['topics_per_page'];
		
		$data['limit'] = $start . ', ' . $end;
		
		$sorts = array(
			'date_add ASC', 'date_add DESC'
		);
		if (isset($this->url['params']['sort']) && in_array(urldecode($this->url['params']['sort']), $sorts)) {
			$data['sort'] = urldecode($this->url['params']['sort']);
		}
		
		$url = SITE_URL . '/' . $this->url['component'];
		foreach ($this->url['params'] as $key => $value) {
			if ($key != 'page') {
				$url .= '/' . $key . '=' . $value;
			}
		}

		$users = array();
		
		$this->data['topics'] = array();
		
		$topics_total = $this->model->get_topics_total();
		
		$topics = $this->model->get_topics($data);
		
		foreach ($topics as $topic) {
			$posts_total = $this->model->get_posts_total(array('topic_id' => $topic['topic_id']));
			
			if ($posts_total) {
				$data = array(
					'topic_id' => $topic['topic_id'],
					'sort'     => 'date_add DESC',
					'limit'    => 1
				);
				
				$last_post_info = array_pop($this->model->get_posts($data));
				
				$user_id = $last_post_info['user_id'];
				$last_post_date_add = $last_post_info['date_add'];
				
			} else {
				$user_id = $topic['user_id'];
				$last_post_date_add = $topic['date_add'];
			}
			
			if (!isset($users[$user_id])) {
				$user_info = $this->model_component_users->get_user_by_id($user_id);
				if ($user_info) {
					$users[$user_id] = $user_info['login'];
				} else {
					$users[$user_id] = 'Deleted';
				}
			}
			
			$last_post = array(
				'username' => $users[$user_id],
				'date_add' => date('H:i d.m.Y', $last_post_date_add)
			);
			
			$this->data['topics'][] = array(
				'name' => $topic['name'],
				'href' => SITE_URL . '/' . $this->url['component'] . '/' . $topic['url'],
				'posts_total' => $posts_total,
				'date_add' => date('H:i d.m.Y', $topic['date_add']),
				'last_post' => $last_post
			);
		}
		
		if (isset($_SESSION['forum_success_register'])) {
			$this->data['success'] = $_SESSION['forum_success_register'];
			unset($_SESSION['forum_success_register']);
		} else {
			$this->data['success'] = false;
		}
		
		$this->load_helper('breadcrumbs');
		$this->helper_breadcrumbs->add('Главная', SITE_URL);
		$this->helper_breadcrumbs->add('Форум', $this->url['component']);
		$this->data['breadcrumbs'] = $this->helper_breadcrumbs->render();
		
		$this->load_helper('pagination');
		$this->helper_pagination->total = $topics_total;
		$this->helper_pagination->page = $page;
		$this->helper_pagination->limit = $this->component_config['topics_per_page'];
		$this->helper_pagination->url = $url . '/page={page}';
		$this->data['pagination'] = $this->helper_pagination->render();
		
		$this->data['href_add_topic'] = SITE_URL . '/' . $this->url['component'] . '/add_topic';
		
		$this->data['page_name'] = $this->component_info['name'];

		$this->page['head'] = $this->add_css_file(SITE_URL . '/modules/components/' . $this->component_info['component'] . '/front_end/views/forum.css');
		$this->page['html'] = $this->load_view();
		return $this->page;
	}
	
	public function action_else() {
		$topic_url = $this->url['actions'][0];
		
		$topic = $this->model->get_topic($topic_url, 'by_url');
		
		$url = SITE_URL . '/' . $this->url['component'] . '/' . $topic_url;
		foreach ($this->url['params'] as $key => $value) {
			if ($key != 'page') {
				$url .= '/' . $key . '=' . $value;
			}
		}
		
		if ($topic) {
			$this->set_global_data('topic_id', $topic['topic_id']);
			
			$users = array();
			
			$this->load_model('component', 'users');
			
			$user = $this->get_global_data('user');
			if ($user) {
				$this->data['show_form'] = true;
			} else {
				$this->data['show_form'] = false;
			}
			
			if (isset($_POST['add_post']) && $this->validate_post()) {
				$post_data = array(
					'topic_id' => $topic['topic_id'],
					'user_id' => $user['id'],
					'text' => $this->helper_validate->safestr($_POST['text'], true)
				);
				
				$this->model->add_post($post_data);
				
				$_SESSION['forum_add_post_success'] = 'Пост добавлен';
				$this->redirect(SITE_URL . $this->url['request_uri']);
			}
			
			if (isset($_SESSION['forum_add_post_success'])) {
				$this->data['success'] = $_SESSION['forum_add_post_success'];
				unset($_SESSION['forum_add_post_success']);
			} elseif (isset($_SESSION['forum_add_topic_success'])) {
				$this->data['success'] = $_SESSION['forum_add_topic_success'];
				unset($_SESSION['forum_add_topic_success']);
			} else {
				$this->data['success'] = false;
			}
			
			// Топик
			$topic_starter = $this->model_component_users->get_user_by_id($topic['user_id']);
			
			if ($topic_starter) {
				$users[$topic['user_id']] = $topic_starter['login'];
				$topic_starter_username = $topic_starter['login'];
			} else {
				$users[$topic['user_id']] = 'Deleted';
				$topic_starter_username = 'Deleted';
			}
			
			$this->data['topic'] = array(
				'topic_id' => $topic['topic_id'],
				'name'     => $topic['name'],
				'user_id'  => $topic['user_id'],
				'username' => $topic_starter_username,
				'text'     => $topic['text'],
				'date_add' => date('H:i d.m.Y', $topic['date_add'])
			);
		
			// Посты
			if (isset($this->url['params']['page']) && $this->url['params']['page'] > 0) {
				$page = (int)$this->url['params']['page'];
			} else {
				$page = 1;
			}
			
			$start = $this->component_config['posts_per_page'] * ($page - 1);
			$end = $page * $this->component_config['posts_per_page'];
		
			$data = array(
				'topic_id' => $topic['topic_id'],
				'sort' => 'date_add ASC',
				'limit' => $start . ', ' . $end
			);
			
			$post_num = ($page - 1) * $this->component_config['posts_per_page'];
			
			$posts_total = $this->model->get_posts_total(array('topic_id' => $topic['topic_id']));
			
			$this->data['posts'] = array();
			
			$posts = $this->model->get_posts($data);
			
			foreach ($posts as $post) {
				$post_num++;
				
				if (!isset($users[$post['user_id']])) {
					$user_info = $this->model_component_users->get_user_by_id($post['user_id']);
					if ($user_info) {
						$users[$post['user_id']] = $user_info['login'];
					} else {
						$users[$post['user_id']] = 'Deleted';
					}
				}
			
				$this->data['posts'][] = array(
					'post_id' => $post['post_id'],
					'num'     => $post_num,
					'user_id' => $post['user_id'],
					'username' => $users[$post['user_id']],
					'text' => $post['text'],
					'date_add' => date('H:i d.m.Y', $post['date_add'])
				);
			}

			$this->load_helper('breadcrumbs');
			$this->helper_breadcrumbs->add('Главная', SITE_URL);
			$this->helper_breadcrumbs->add($this->component_info['name'], SITE_URL . '/' . $this->url['component']);
			$this->helper_breadcrumbs->add($topic['name'], SITE_URL . '/' . $this->url['component'] . '/' . $topic['url']);
			$this->data['breadcrumbs'] = $this->helper_breadcrumbs->render();
			
			$this->load_helper('pagination');
			$this->helper_pagination->total = $posts_total;
			$this->helper_pagination->page = $page;
			$this->helper_pagination->limit = $this->component_config['posts_per_page'];
			$this->helper_pagination->url = $url . '/page={page}';
			$this->data['pagination'] = $this->helper_pagination->render();
			
			$this->data['errors'] = $this->errors;
			$this->data['page_name'] = $this->data['topic']['name'];
			$this->data['href_login'] = SITE_URL . '/' . $this->url['component'] . '/login';
			$this->data['href_register'] = SITE_URL . '/' . $this->url['component'] . '/register';
			
			$this->page['head'] = $this->add_css_file(SITE_URL . '/modules/components/' . $this->component_info['component'] . '/front_end/views/forum.css');
			$this->page['html'] = $this->load_view('topic');
			return $this->page;
		} else {
			return $this->action_404('404');
		}
	}
	
	public function action_add_topic() {
		$user = $this->get_global_data('user');
	
		if (isset($_POST['add_topic']) && $this->validate_topic()) {
			$topic_url = $this->str2url($_POST['topic_name']);
			
			$topic_data = array(
				'user_id' => $user['id'],
				'name'    => $this->helper_validate->safestr($_POST['topic_name']),
				'text'    => $this->helper_validate->safestr($_POST['topic_text'], true),
				'url'     => $topic_url
			);
			
			$this->model->add_topic($topic_data);
			
			$_SESSION['forum_add_topic_success'] = 'Топик создан';
			$this->redirect(SITE_URL . '/' . $this->url['component'] . '/' . $topic_url);
		}
		
		if ($user) {
			$this->data['show_form'] = true;
		} else {
			$this->data['show_form'] = false;
		}
		
		if (isset($_POST['topic_name'])) {
		
			$this->data['topic_name'] = $_POST['topic_name'];
		} else {
			$this->data['topic_name'] = '';
		}
		
		if (isset($_POST['topic_text'])) {
			$this->data['topic_text'] = $_POST['topic_text'];
		} else {
			$this->data['topic_text'] = '';
		}
		
		$this->data['href_login'] = SITE_URL . '/' . $this->url['component'] . '/login';
		$this->data['href_register'] = SITE_URL . '/' . $this->url['component'] . '/register';
		
		$this->data['page_name'] = 'Добавление топика';
		
		$this->load_helper('breadcrumbs');
		$this->helper_breadcrumbs->add('Главная', SITE_URL);
		$this->helper_breadcrumbs->add($this->component_info['name'], SITE_URL . '/' . $this->url['component']);
		$this->helper_breadcrumbs->add('Добавление топика', SITE_URL . '/' . $this->url['component'] . '/add_topic');
		$this->data['breadcrumbs'] = $this->helper_breadcrumbs->render();
		
		$this->data['errors'] = $this->errors;
		
		$this->page['head'] = $this->add_css_file(SITE_URL . '/modules/components/' . $this->component_info['component'] . '/front_end/views/forum.css');
		$this->page['html'] = $this->load_view('add_topic');
		return $this->page;
	}
	
	public function action_login() {
		$this->load_model('component', 'users');
		if ($this->get_global_data('user')) {
			$this->redirect(SITE_URL . '/' . $this->url['component']);
		}
		
		if (isset($_POST['auth']) && $this->validate_login()) {
			if ($this->model_component_users->authorisation($_POST['login'], $_POST['password'])) {
				$user = $this->model_component_users->get_user($_POST['login']);
				
				$_SESSION['auth'] = 1;
				$_SESSION['login'] = $user['login'];
				$_SESSION['access'] = (int)$user['access_level'];
				
				$this->redirect(SITE_URL . '/' . $this->url['component']);
			}
		}
		
		$this->load_helper('breadcrumbs');
		$this->helper_breadcrumbs->add('Главная', SITE_URL);
		$this->helper_breadcrumbs->add($this->component_info['name'], SITE_URL . '/' . $this->url['component']);
		$this->helper_breadcrumbs->add('Авторизация', SITE_URL . '/' . $this->url['component'] . '/login');
		$this->data['breadcrumbs'] = $this->helper_breadcrumbs->render();
		
		$this->data['errors'] = $this->errors;
		$this->data['page_name'] = 'Авторизация';
		$this->page['head'] = $this->add_css_file(SITE_URL . '/modules/components/' . $this->component_info['component'] . '/front_end/views/forum.css');
		$this->page['html'] = $this->load_view('login');
		return $this->page;
	}
	
	public function action_register() {
		$this->load_model('component', 'users');
		if ($this->get_global_data('user')) {
			$this->redirect(SITE_URL . '/' . $this->url['component']);
		}
		
		if (isset($_POST['register']) && $this->validate_register()) {
			$data = array(
				'login' => strip_tags($_POST['login']),
				'email' => $_POST['email'],
				'password' => $_POST['password'],
				'access_level' => 0
			);
			
			$user_id = $this->model_component_users->add_user($data);
			
			$user = $this->model_component_users->get_user_by_id($user_id);
			
			$_SESSION['auth'] = 1;
			$_SESSION['login'] = $user['login'];
			$_SESSION['access'] = 0;
			
			$_SESSION['forum_success_register'] = 'Регистрация завершена';
			$this->redirect(SITE_URL . '/' . $this->url['component']);
		}
		
		if (isset($_POST['login'])) {
			$this->data['login'] = strip_tags($_POST['login']);
		} else {
			$this->data['login'] = '';
		}
		
		if (isset($_POST['email'])) {
			$this->data['email'] = $_POST['email'];
		} else {
			$this->data['email'] = '';
		}
		
		$this->data['src_captcha'] = SITE_URL . '/user_cms/helpers/captcha.php';
		
		$this->load_helper('breadcrumbs');
		$this->helper_breadcrumbs->add('Главная', SITE_URL);
		$this->helper_breadcrumbs->add($this->component_info['name'], SITE_URL . '/' . $this->url['component']);
		$this->helper_breadcrumbs->add('Регистрация', SITE_URL . '/' . $this->url['component'] . '/register');
		$this->data['breadcrumbs'] = $this->helper_breadcrumbs->render();
		
		$this->data['errors'] = $this->errors;
		$this->data['page_name'] = 'Регистрация';
		$this->page['head'] = $this->add_css_file(SITE_URL . '/modules/components/' . $this->component_info['component'] . '/front_end/views/forum.css');
		$this->page['html'] = $this->load_view('register');
		return $this->page;
	}
	
	private function validate_post() {
		$this->load_helper('validate');
		
		if (!isset($_POST['text']) || empty($_POST['text'])) {
			$this->errors[]  = 'Введите текст сообщения';
		}
		
		if (!$this->get_global_data('user')) {
			$this->errors[] = 'Войдите в аккаунт';
		}
		
		if ($this->errors) {
			return false;
		} else {
			return true;
		}
	}
	
	private function validate_topic() {
		$this->load_helper('validate');
		
		if (!isset($_POST['topic_name']) || empty($_POST['topic_name'])) {
			$this->errors[] = 'Введите название топика';
		} else {
			$topic_name = $this->helper_validate->safestr($_POST['topic_name']);
			
			if (mb_strlen($topic_name, 'UTF-8') > 60) {
				$this->errors[] = 'Название топика должно содержать не больше 60 символов';
			} else {
				$url = $this->str2url($topic_name);
				if ($this->model->get_topic($url, 'by_url')) {
					$this->errors[] = 'Топик с таким названием уже существует';
				}
			} 
		}
		
		if (!isset($_POST['topic_text']) || empty($_POST['topic_text'])) {
			$this->errors[] = 'Введите текст топика';
		}
		
		if ($this->errors) {
			return false;
		} else {
			return true;
		}
	}
	
	private function validate_login() {
		if (!isset($_POST['login']) || empty($_POST['login']) || !isset($_POST['password']) || empty($_POST['password'])) {
			$this->errors[] = 'Неверное имя пользователя и/или пароль';
			return false;
		} else {
			return true;
		}
	}
	
	private function validate_register() {
		$this->load_helper('validate');
		
		if (!isset($_POST['email']) || !$this->helper_validate->email($_POST['email'])) {
			$this->errors[] = 'Укажите Email';
		}
		
		if (!isset($_POST['login']) || empty($_POST['login'])) {
			$this->errors[] = 'Укажите логин';
		} elseif ($this->model_component_users->get_user($_POST['login'])) {
			$this->errors[] = 'Пользователь с таким логином уже существует';
		} else {
			if (!preg_match('/^[a-zA-Z_\-0-9]{3,32}$/', $_POST['login'])) {
				$this->errors[] = 'Логин может содержать только латинские символы, цифры, дефис, нижнее подчеркивание';
			}
			if (mb_strlen($_POST['login'],'UTF-8') < 3 || mb_strlen($_POST['login'],'UTF-8') > 32) {
				$this->errors[] = 'Логин должен быть от 3 до 32 символов';
			}
		}
		
		if (!isset($_SESSION['captcha']) || !isset($_POST['captcha']) || empty($_POST['captcha']) || $_SESSION['captcha'] !== $_POST['captcha']) {
			$this->errors[] = 'Неверные символы';
		}
		
		if (!isset($_POST['password']) || empty($_POST['password'])) {
			$this->errors[] = 'Укажите пароль';
		} elseif (mb_strlen($_POST['password'],'UTF-8') < 6 || mb_strlen($_POST['password'],'UTF-8') > 16) {
			$this->errors[] = 'Пароль должен быть от 6 до 16 символов';
		} elseif (!isset($_POST['password_2']) || empty($_POST['password_2']) || $_POST['password'] !== $_POST['password_2']) {
			$this->errors[] = 'Пароли не совпадают';
		}
		
		if ($this->errors) {
			return false;
		} else {
			return true;
		}
	}
}